/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Producer.h"
#include "AbstractTranslation.h"
#include "FunctionTranslation.h"
#include <ocstring.h>
#include "ByteArray.h"
#include "Msg.h"
#include "Node.h"
#include "Selector.h"
#include "ArgumentList.h"

@implementation FunctionTranslation : AbstractTranslation {
	id functionName;
	id functionArgumentList;
}
+ name:aFunctionName args:anArgumentList {
	self = [super new];
	functionName = aFunctionName;
	functionArgumentList = anArgumentList;
	return self;
}
- genReceiver:aReceiver selector:aSelector {
	id arg; unsigned argNumber = 0; 
	[functionName gen]; gc('(');
	for (arg = functionArgumentList; arg; arg = [arg successor]) {
		STR name = [arg str]; 
		if (argNumber != 0) gc(',');
		if (*name == '%') {
			unsigned i = atoi(name+1);
			if (i == 0) [aReceiver gen];
			else if (i >= [aSelector size])
				wer("argument offset %d out of range", i);
			else [[[aSelector at:i-1] argument] gen];
		} else if (argNumber == 0) [aReceiver gen];
		else [[[aSelector at:argNumber-1] argument] gen];
		argNumber++;
	}
	gc(')'); return self;
}
- (STR)str { return [functionName str]; }
#define MAXARRAY 2048
- asTypedByteArray { char buf[MAXARRAY]; id arg; 
	strcpy(buf, [functionName str]);
	for (arg = functionArgumentList; arg; arg = [arg successor]) {
		sprintf(buf+strlen(buf), "(%s)%s ", 
			[[arg argumentType] str], [[arg argumentName] str]);;
	}
	return [ByteArray str:buf];
}
@end

