/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Producer.h"
#include <ocstring.h>
#include "ByteArray.h"
#include <set.h>
#include <ordcltn.h>
#include "Identifier.h"

	IMPORT id identifierTranslator;

@implementation Identifier:ByteArray
	{ id translation, type; }

+ name:aByteArray 
	{ return [self str:[aByteArray str]]; }
+ str:(STR)aString { 
	self = [super str:aString];
	type = types.UNKNOWN;
	translation = [identifierTranslator find:self];
	return self;
}
+ uniqueIdentifier:(STR)aString { static int uniqueness = 0;
	return [self sprintf:"%s%d", aString, uniqueness++];
}
- gen {
	if (translation) [translation gen];
	else gs([self str]); 
	return self; 
}
- genDeclaration {
	if (translation) [translation genDeclaration];
	else {
		if (type == nil) gs("<nil>"); else [type gen];
		gc(' '); gs([self str]); gc(';');
	}
	return self;
}
- type { 
	if (translation) return type = [translation type];
#if 0
	dbg("%s: (%s)%s\n", NAMEOF(self), [type str], [self str]);
	if (type == types.UNKNOWN) [self type:types.ID rule:"default: first use"];
#endif
	return type; 
}
- type:aType rule:(STR)aString {
	if (translation && type != aType) {
		info("attempt to change type of translated symbol %s ignored (%s)",
			[self str], aString);
		return self;
	}
	if (aType == nil)
		return [self error:"nil type"]; 
	if (type != types.UNKNOWN && aType != type) {
		wer("%s %s; tried to change type from %s to %s ignored (%s)",
			NAMEOF(self), [self str], [type str], [aType str], aString);
	} else {
		info("type of %s is (%s) (%s)\n", [self str], [aType str], aString);
		type = aType;
	}
	return self;
}
- free { return nil; }

@end

